@@ -5,7 +5,6 @@ from multiprocessing import Pool
 
 from napoleontoolbox.file_saver import dropbox_file_saver
 from napoleontoolbox.signal import signal_generator
-from napoleontoolbox.connector import napoleon_connector
 from napoleontoolbox.signal import signal_utility
 import json
 
@@ -211,7 +210,7 @@ def pair_handling_message(fetcher, msg, compute_minute_signal=True, compute_day_
     volu_dict[trade_time_unix_timestamp_minute] = actual_volu + float(msg['q'])
 
 class RealTimeSignalEnsemblingParalleLauncher():
-    def __init__(self, min_calib_starting_date=None, min_calib_running_date=None, day_calib_starting_date=None, day_calib_running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, min_calib_selected_algo='', day_calib_selected_algo='', user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl', backed_up_signal_histo_suffix='backed_up_signal_histo.pkl', load_daily_histo = True, from_cryptocompare=False):
+    def __init__(self, min_calib_starting_date=None, min_calib_running_date=None, day_calib_starting_date=None, day_calib_running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, min_calib_selected_algo='', day_calib_selected_algo='', user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl', backed_up_day_signal_histo_suffix='backed_up_day_signal_histo.pkl', backed_up_min_signal_histo_suffix='backed_up_min_signal_histo.pkl', load_daily_histo = True, from_cryptocompare=False, daily_ia_lookback_window=21, minutely_ia_lookback_window=21):
         self.min_calib_starting_date = min_calib_starting_date
         self.min_calib_running_date = min_calib_running_date
 
@@ -237,17 +236,24 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.aws_client = napoleon_s3_connector.NapoleonS3Connector(AWS_PUBLIC,AWS_SECRET,region=AWS_REGION)
         self.pair = pair
         self.generic_columns = ['ts', 'open', 'high', 'low', 'close', 'volumefrom']
-        self.minute_df = pd.DataFrame(columns=self.generic_columns)
 
+        self.minute_df = pd.DataFrame(columns=self.generic_columns)
         self.hour_df = pd.DataFrame(columns=self.generic_columns)
         self.day_df = pd.DataFrame(columns=self.generic_columns)
 
+        self.minute_signal_df = None
+        self.hour_signal_df = None
+        self.day_signal_df = None
+
         self.minute_np = None
         self.minute_signal_np = None
         self.hour_np = None
         self.day_np = None
         self.day_signal_np = None
 
+        self.day_discretized_signal_np = None
+        self.minute_discretized_signal_np = None
+
         self.pair_dict = {}
         self.pair_dict['high'] = {}
         self.pair_dict['low'] = {}
@@ -288,7 +294,8 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print('days_selected algos')
         print(self.day_calib_list_pkl_file_name)
 
-        self.backed_up_signal_histo_suffix = backed_up_signal_histo_suffix
+        self.backed_up_day_signal_histo_suffix = backed_up_day_signal_histo_suffix
+        self.backed_up_min_signal_histo_suffix = backed_up_min_signal_histo_suffix
 
 
         if self.dropbox_backup:
@@ -348,8 +355,19 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.from_cryptocompare = from_cryptocompare
         if self.load_daily_histo:
             print('loading daily historical signals')
-            self.reload_daily_histo(from_cryptocompare = self.from_cryptocompare)
+            self.reload_daily_histo(from_cryptocompare = self.from_cryptocompare, running_date = datetime.now())
 
+        self.daily_ia_lookback_window = daily_ia_lookback_window
+        self.minutely_ia_lookback_window = minutely_ia_lookback_window
+        self.on_the_fly_discrete_minute_signals =[]
+        self.on_the_fly_discrete_day_signals =[]
+
+    def compare_live_vs_histo(self, live, histo):
+        comparison_df = pd.merge(live, histo, how='inner', on=None, left_on=None, right_on=None,
+                                 left_index=True, right_index=True, sort=True,
+                                 suffixes=('_live', '_histo'), copy=True, indicator=False,
+                                 validate=None)
+        return comparison_df
 
     def get_aws_last_past_minutes(self, nb_last_hours = 24):
         date_now = datetime.utcnow()
@@ -493,10 +511,94 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(aggregated_df.shape)
         return aggregated_df
 
-    def recompute_daily_histo(self, from_cryptocompare = False):
+
+    def recompute_minute_histo(self, from_cryptocompare = False, running_date = None, nb_last_hours=24):
+        if from_cryptocompare:
+            starting_date = running_date - timedelta(hours = nb_last_hours)
+            freqly_pkl_file_name_suffix = '_min_tmp.pkl'
+            mins_df = napoleon_connector.fetch_crypto_minutely_data(ssj=self.underlying, local_root_directory=self.local_root_directory,
+                                                                   minutely_return_pkl_filename_suffix=freqly_pkl_file_name_suffix,
+                                                                   daily_crypto_starting_day=starting_date,
+                                                                   daily_crypto_ending_day=running_date,
+                                                                   refetch_all=True)
+            print('mins size before filternig')
+            print(mins_df.shape)
+            mins_df=mins_df[mins_df.index >= starting_date]
+            mins_df = mins_df[mins_df.index <= running_date]
+            print('mins size after filternig')
+            print(mins_df.shape)
+            self.recreate_minutely_histo(mins_df, running_date)
+        else:
+            print('other provider not implemented')
+
+    def recreate_minutely_histo(self, minutely_df, up_to_date):
+        cumulated_signals = None
+        print('Number of minute signals to aggregate '+str(len(self.min_calib_signals_list)))
+        skipping_size = self.day_max_lookback_window
+        mapping_dictionary = {}
+        counter = 1
+        for me_signal in self.min_calib_signals_list:
+            print('backtesting '+str(counter)+' over '+str(len(self.min_calib_signals_list)))
+            # we have to reload the file each time
+            freqly_df = minutely_df.copy()
+            ## idiosyncratic run itself
+            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
+            params = json.loads(run_json_string)
+            normalization = params['normalization']
+            transaction_costs = params['transaction_costs']
+            trigger = params['trigger']
+            signal_type = params['signal_type']
+            if normalization and not signal_generator.is_signal_continuum(signal_type):
+                return
+            params = json.loads(run_json_string)
+            salty = str(int(hashlib.sha1(run_json_string.encode('utf-8')).hexdigest(), 16) % (10 ** 8))
+            params = json.loads(run_json_string)
+            signal_column = params['signal_type'] + str(params['trigger']) + salty
+            mapping_dictionary[me_signal]=signal_column
+            if signal_type == 'long_only':
+                freqly_df['signal']=1.
+            else:
+                lookback_window = params['lookback_window']
+                if skipping_size < lookback_window:
+                    raise Exception('The skipping size must be greater than the lookback window')
+                # kwargs = {**generics, **idios}
+                signal_generation_method_to_call = getattr(signal_generator, signal_type)
+                freqly_df = signal_utility.roll_wrapper(freqly_df, lookback_window, skipping_size,
+                                          lambda x: signal_generation_method_to_call(data=x, **params), trigger)
+                freqly_df, _ = signal_utility.reconstitute_signal_perf(data = freqly_df, transaction_cost = transaction_costs, normalization = normalization)
+
+            if cumulated_signals is None:
+                freqly_df['signal_not_shifted'] = freqly_df['signal'].shift(-1)
+                cumulated_signals = freqly_df[['close','high','low','open','volumefrom','signal_not_shifted']].copy()
+                cumulated_signals = cumulated_signals.rename(columns={'signal_not_shifted': signal_column})
+            else :
+                freqly_df['signal_not_shifted'] = freqly_df['signal'].shift(-1)
+                cumulated_signals[signal_column]=freqly_df['signal_not_shifted'].copy()
+            counter=counter+1
+
+        print('recomputation done')
+        print(cumulated_signals.shape)
+        cumulated_signals['date'] = pd.to_datetime(cumulated_signals.index)
+        cumulated_signals['ts'] = cumulated_signals['date'].values.astype(np.int64) // 10 ** 9
+
+        print('discretizing the signal')
+        discrete_daily_signals = self.minutely_apply_standard_mean_two_states_discretization(signals_df=cumulated_signals, s=self.minutely_ia_lookback_window)
+        print('discretization done')
+        cumulated_signals['trend']=discrete_daily_signals
+
+        backed_up_min_signal_histo_name =  up_to_date.strftime('%d_%b_%Y_%H')+'_'+self.underlying+'_'+self.backed_up_min_signal_histo_suffix
+        saving_path = self.local_root_directory+backed_up_min_signal_histo_name
+        print('saving minute histo recomputation to '+backed_up_min_signal_histo_name)
+        cumulated_signals.to_pickle(saving_path)
+
+        self.minute_discretized_signal_np = discrete_daily_signals
+        discretized_saving_path = saving_path.replace('pkl','')
+        print('saving minute histo discretized recomputation to '+discretized_saving_path)
+        np.save(discretized_saving_path, self.minute_discretized_signal_np)
+
+    def recompute_daily_histo(self, from_cryptocompare = False, running_date=None):
         if from_cryptocompare:
             print('recomputing historical up to '+str(self.day_max_lookback_window))
-            running_date = datetime.now()
             starting_date = running_date - timedelta(days=2*self.day_max_lookback_window)
             daily_df = napoleon_connector.fetch_crypto_daily_data(ssj=self.underlying, local_root_directory=self.local_root_directory,
                                                                    daily_return_pkl_filename_suffix='_daily_temp.pkl',
@@ -511,12 +613,12 @@ class RealTimeSignalEnsemblingParalleLauncher():
             print('data refetched')
             self.recreate_daily_histo(daily_df,running_date)
         else :
-            print('not enough ohlc binance history')
+            print('other provider not implemented')
 
 
     def recreate_daily_histo(self, daily_df, up_to_date):
         cumulated_signals = None
-        print('Number of signals to aggregate '+str(len(self.day_calib_signals_list)))
+        print('Number of day signals to aggregate '+str(len(self.day_calib_signals_list)))
         skipping_size = self.day_max_lookback_window
         mapping_dictionary = {}
         counter = 1
@@ -563,29 +665,99 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(cumulated_signals.shape)
         cumulated_signals['date'] = pd.to_datetime(cumulated_signals.index)
         cumulated_signals['ts'] = cumulated_signals['date'].values.astype(np.int64) // 10 ** 9
-
         up_to_date = up_to_date - timedelta(days=1)
         cumulated_signals=cumulated_signals[cumulated_signals.index<=up_to_date]
-
-        backed_up_signal_histo_name =  up_to_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_signal_histo_suffix
+        backed_up_signal_histo_name =  up_to_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_day_signal_histo_suffix
+        print('discretizing the signal')
+        discrete_daily_signals = self.daily_apply_standard_mean_two_states_discretization(signals_df=cumulated_signals, s=self.daily_ia_lookback_window)
+        print('discretization done')
+        cumulated_signals['trend']=discrete_daily_signals
         saving_path = self.local_root_directory+backed_up_signal_histo_name
         print('saving histo recomputation to '+backed_up_signal_histo_name)
         cumulated_signals.to_pickle(saving_path)
+        self.day_discretized_signal_np = discrete_daily_signals
+        discretized_saving_path = saving_path.replace('pkl','')
+        print('saving day histo discretized recomputation to '+discretized_saving_path)
+        np.save(discretized_saving_path, self.day_discretized_signal_np)
 
 
-    def reload_minute_histo(self, from_cryptocompare = True, nb_last_hours=24):
-        if not from_cryptocompare :
-            raise Exception('not implemented yet')
-        running_date = datetime.now()
-        starting_date = running_date - timedelta(hours = nb_last_hours)
-        freqly_pkl_file_name_suffix = '_min_tmp.pkl'
-        mins_df = napoleon_connector.fetch_crypto_minutely_data(ssj=self.underlying, local_root_directory=self.local_root_directory,
-                                                               minutely_return_pkl_filename_suffix=freqly_pkl_file_name_suffix,
-                                                               daily_crypto_starting_day=starting_date,
-                                                               daily_crypto_ending_day=running_date,
-                                                               refetch_all=True)
-        return mins_df
+    def daily_apply_standard_mean_two_states_discretization_last_signal(self):
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        prediction_index = self.day_signal_df.shape[0]
+        starting_train = prediction_index - 1 - self.daily_ia_lookback_window
+        stopping_train = prediction_index - 1
 
+        local_signal_df = self.day_signal_df[signal_columns]
+        full_y_true = self.day_signal_df['close'].pct_change().fillna(0.)
+
+        local_rolling_np = local_signal_df.values[starting_train:stopping_train, :]
+
+        y_train_true = full_y_true[starting_train:stopping_train]
+        y_train_pred = np.mean(local_rolling_np, axis=1)
+
+        splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
+            y_train_pred, y_train_true)
+        y_next_pred = local_signal_df.values[prediction_index, :].mean()
+        y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+        return y_next_pred_discrete
+
+    def minutely_apply_standard_mean_two_states_discretization_last_signal(self):
+        signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+        prediction_index = self.minute_signal_df.shape[0]
+        starting_train = prediction_index - 1 - self.minutely_ia_lookback_window
+        stopping_train = prediction_index - 1
+
+        local_signal_df = self.minute_signal_df[signal_columns]
+        full_y_true = self.minute_signal_df['close'].pct_change().fillna(0.)
+
+        local_rolling_np = local_signal_df.values[starting_train:stopping_train, :]
+
+        y_train_true = full_y_true[starting_train:stopping_train]
+        y_train_pred = np.mean(local_rolling_np, axis=1)
+
+        splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
+            y_train_pred, y_train_true)
+        y_next_pred = local_signal_df.values[prediction_index, :].mean()
+        y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+        return y_next_pred_discrete
+
+    def daily_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21):
+        day_signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        signals_np = signals_df[day_signal_columns].values#[:, len(self.generic_columns):]
+        full_y_true = signals_df['close'].pct_change().fillna(0.)
+        y_pred_discretized = np.zeros(full_y_true.shape)
+        for row_index in range(s + 1, signals_np.shape[0]):
+            starting_train = row_index - 1 - s
+            stopping_train = row_index - 1
+            prediction = row_index
+            rolling_np = signals_np[starting_train:stopping_train, :]
+            y_train_true = full_y_true[starting_train:stopping_train]
+            y_train_pred = np.mean(rolling_np, axis=1)
+            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
+                y_train_pred, y_train_true)
+            y_next_pred = signals_np[row_index, :].mean()
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+            y_pred_discretized[prediction] = y_next_pred_discrete
+        return y_pred_discretized
+
+    def minutely_apply_standard_mean_two_states_discretization(self, signals_df=None, s=21):
+        minute_signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+        signals_np = signals_df[minute_signal_columns].values#[:, len(self.generic_columns):]
+        full_y_true = signals_df['close'].pct_change().fillna(0.)
+        y_pred_discretized = np.zeros(full_y_true.shape)
+        for row_index in range(s + 1, signals_np.shape[0]):
+            starting_train = row_index - 1 - s
+            stopping_train = row_index - 1
+            prediction = row_index
+            rolling_np = signals_np[starting_train:stopping_train, :]
+            y_train_true = full_y_true[starting_train:stopping_train]
+            y_train_pred = np.mean(rolling_np, axis=1)
+            splitting_quantile_threshold, splitting_threshold, sharpe_strat, sharpe_under = signal_utility.find_best_two_states_discretization(
+                y_train_pred, y_train_true)
+            y_next_pred = signals_np[row_index, :].mean()
+            y_next_pred_discrete = signal_utility.apply_discretization_two_states(y_next_pred, splitting_threshold)
+            y_pred_discretized[prediction] = y_next_pred_discrete
+        return y_pred_discretized
 
     def reload_hour_histo(self, from_cryptocompare=False, nb_last_days = 1):
         if not from_cryptocompare:
@@ -601,10 +773,10 @@ class RealTimeSignalEnsemblingParalleLauncher():
                                                                  refetch_all=True)
         return hours_df
 
-    def reload_daily_histo(self, from_cryptocompare = False):
-        upto_date = datetime.now() - timedelta(days=1)
+    def reload_daily_histo(self, from_cryptocompare = False,running_date = None):
+        running_date = running_date - timedelta(days=1)
         if from_cryptocompare :
-            backed_up_signal_histo_name =  upto_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_signal_histo_suffix
+            backed_up_signal_histo_name =  running_date.strftime('%d_%b_%Y')+'_'+self.underlying+'_'+self.backed_up_day_signal_histo_suffix
             saving_path = self.local_root_directory+backed_up_signal_histo_name
             print('loading histo recomputation to '+backed_up_signal_histo_name)
             updated_df = pd.read_pickle(saving_path)
@@ -620,7 +792,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
             updated_df =self.get_aws_last_past_days()
             updated_df['date'] = pd.to_datetime(updated_df.index)
             updated_df['ts'] = updated_df['date'].values.astype(np.int64) // 10 ** 9
-            updated_df = updated_df[updated_df.index <= upto_date]
+            updated_df = updated_df[updated_df.index <= running_date]
             updated_df=updated_df.sort_index()
             signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
             me_columns = self.generic_columns + signal_columns
@@ -631,6 +803,36 @@ class RealTimeSignalEnsemblingParalleLauncher():
             self.day_df = day_df
         return updated_df
 
+    def reload_minute_histo(self, from_cryptocompare = True, running_date = None, nb_last_hours=24):
+        if from_cryptocompare :
+            backed_up_min_signal_histo_name =  running_date.strftime('%d_%b_%Y_%H')+'_'+self.underlying+'_'+self.backed_up_min_signal_histo_suffix
+            saving_path = self.local_root_directory+backed_up_min_signal_histo_name
+            print('loading histo recomputation to '+backed_up_min_signal_histo_name)
+            updated_df = pd.read_pickle(saving_path)
+            updated_df=updated_df.sort_index()
+            signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+            me_columns = self.generic_columns + signal_columns
+            updated_df = updated_df[me_columns].copy()
+            minute_df = updated_df[self.generic_columns].copy()
+            self.minute_signal_np = updated_df.values
+            self.minute_np = minute_df.values
+            self.minute_df = minute_df
+        else:
+            updated_df = self.get_aws_last_past_minutes(nb_last_hours=nb_last_hours)
+            updated_df['date'] = pd.to_datetime(updated_df.index)
+            updated_df['ts'] = updated_df['date'].values.astype(np.int64) // 10 ** 9
+            updated_df = updated_df[updated_df.index <= running_date]
+            updated_df=updated_df.sort_index()
+            signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+            me_columns = self.generic_columns + signal_columns
+            updated_df = updated_df[me_columns].copy()
+            minute_df = updated_df[self.generic_columns].copy()
+            self.minute_signal_np = updated_df.values
+            self.minute_np = minute_df.values
+            self.minute_df = minute_df
+        return updated_df
+
+
 
     def start_fetching_pair(self):
         bm = BinanceSocketManager(self.client)
@@ -670,6 +872,67 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.day_df = updated_df
         self.last_day = max(self.day_df.index)
 
+    def update_signal_day_dataframe(self):
+        print('updating signal day df')
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        me_columns = self.generic_columns + signal_columns
+
+        updated_df=pd.DataFrame(data=self.day_signal_np, columns = me_columns, index=range(len(self.day_signal_np)))
+        # updated_df['ts']=updated_df['ts'].astype(int)
+        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
+        # updated_df = updated_df.astype({'day_ts': 'int64'})
+        # if updated_df['day_ts'].nunique()>1:
+        #     print('too many days saved together : investigate')
+        # day_timestamp = updated_df['day_ts'].max()
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        me_ts =updated_df['ts']
+        me_day_ts = me_ts-me_ts%(3600*24)
+        me_day_ts = me_day_ts.astype({'ts': 'int64'})
+        if me_day_ts.nunique()>1:
+            print('too many days saved together : investigate')
+        day_timestamp = me_day_ts.max()
+
+        day_dt_object = datetime.utcfromtimestamp(day_timestamp)
+        filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y')+'_dayohlc' #str(hourr_timestamp)
+        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+        updated_df = updated_df.sort_values(by=['ts'])
+        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+        updated_df = updated_df.set_index(updated_df['date'])
+        updated_df = updated_df.drop(columns=['date'])
+        self.day_signal_df = updated_df.copy()
+        return filename
+
+    def update_signal_minute_dataframe(self):
+        print('updating 60 minutes hour df')
+        signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+        me_columns = self.generic_columns + signal_columns
+        updated_df=pd.DataFrame(data=self.minute_signal_np, columns = me_columns, index=range(len(self.minute_signal_np)))
+        #updated_df['ts']=updated_df['ts'].astype(int)
+        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        # updated_df = updated_df.astype({'hour_ts': 'int64'})
+        # if updated_df['hour_ts'].nunique()>1:
+        #     print('too many hours saved together : investigate')
+        # hour_timestamp = updated_df['hour_ts'].max()
+
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        me_ts =updated_df['ts']
+        me_hour_ts = me_ts-me_ts%3600
+        me_hour_ts = me_hour_ts.astype({'ts': 'int64'})
+        if me_hour_ts.nunique()>1:
+            print('too many days saved together : investigate')
+        hour_timestamp = me_hour_ts.max()
+
+        hour_dt_object = datetime.utcfromtimestamp(hour_timestamp)
+        filename = self.pair+'_'+hour_dt_object.strftime('%d_%b_%Y_%H') #str(hourr_timestamp)
+        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+        updated_df = updated_df.sort_values(by=['ts'])
+        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+        updated_df = updated_df.set_index(updated_df['date'])
+        updated_df = updated_df.drop(columns=['date'])
+        self.minute_signal_df = updated_df.copy()
+        return filename
+
     def update_minute_dataframe(self):
         print('updating minute df')
         if len(self.minute_np.shape)>1:
@@ -685,9 +948,7 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.minute_df = updated_df
         self.last_minute = max(self.minute_df.index)
 
-
-
-    def upload_last_day_24_hours_signals_dataframe(self):
+    def update_hour_dataframe(self):
         print('updating 24 hours day df')
         updated_df=pd.DataFrame(data=self.hour_np, columns = self.generic_columns, index=range(len(self.hour_np)))
         # updated_df = updated_df.astype(
@@ -717,76 +978,41 @@ class RealTimeSignalEnsemblingParalleLauncher():
         updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
         updated_df = updated_df.set_index(updated_df['date'])
         updated_df = updated_df.drop(columns=['date'])
-        self.hour_df = updated_df
+        self.hour_df = updated_df.copy()
+        return filename
+
+    def upload_last_day_24_hours_signals_dataframe(self):
+        filename = self.update_hour_dataframe()
         local_path = self.local_root_directory + filename
         self.hour_df.to_csv(local_path)
-        print('uploading last day file '+local_path)
+        print('uploading all hours last day file '+local_path)
         self.aws_client.upload_file(self.bucket,local_path)
+        return
 
     def upload_last_hour_60_minutes_signals_dataframe(self):
-        print('updating 60 minutes hour df')
-        signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
-        me_columns = self.generic_columns + signal_columns
-        updated_df=pd.DataFrame(data=self.minute_signal_np, columns = me_columns, index=range(len(self.minute_signal_np)))
-        #updated_df['ts']=updated_df['ts'].astype(int)
-        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        # updated_df = updated_df.astype({'hour_ts': 'int64'})
-        # if updated_df['hour_ts'].nunique()>1:
-        #     print('too many hours saved together : investigate')
-        # hour_timestamp = updated_df['hour_ts'].max()
-
-        updated_df['ts'] = updated_df['ts'].astype(int)
-        me_ts =updated_df['ts']
-        me_hour_ts = me_ts-me_ts%3600
-        me_hour_ts = me_hour_ts.astype({'ts': 'int64'})
-        if me_hour_ts.nunique()>1:
-            print('too many days saved together : investigate')
-        hour_timestamp = me_hour_ts.max()
-
-        hour_dt_object = datetime.utcfromtimestamp(hour_timestamp)
-        filename = self.pair+'_'+hour_dt_object.strftime('%d_%b_%Y_%H') #str(hourr_timestamp)
-        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
-        updated_df = updated_df.sort_values(by=['ts'])
-        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
-        updated_df = updated_df.set_index(updated_df['date'])
-        updated_df = updated_df.drop(columns=['date'])
-        self.minute_df = updated_df
+        filename = self.update_signal_minute_dataframe()
         local_path = self.local_root_directory + filename
-        self.minute_df.to_csv(local_path)
-        print('uploading last hour file '+local_path)
+        self.minute_signal_df.to_csv(local_path)
+        print('uploading all minutes last hour file '+local_path)
         self.aws_client.upload_file(self.bucket,local_path)
 
+        aggregated_local_path = local_path + 'discrete_aggregate'
+        print('uploading all discrete aggregated minutes last hour file '+aggregated_local_path)
+        pd.DataFrame(self.on_the_fly_discrete_minute_signals).to_csv(aggregated_local_path)
+        self.aws_client.upload_file(self.bucket,aggregated_local_path)
+        return
+
     def upload_days_signals_dataframe(self):
-        print('updating day df')
-        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
-        me_columns = self.generic_columns + signal_columns
-        updated_df=pd.DataFrame(data=self.day_signal_np, columns = me_columns, index=range(len(self.day_signal_np)))
-        # updated_df['ts']=updated_df['ts'].astype(int)
-        # updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
-        # updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
-        # updated_df = updated_df.astype({'day_ts': 'int64'})
-        # if updated_df['day_ts'].nunique()>1:
-        #     print('too many days saved together : investigate')
-        # day_timestamp = updated_df['day_ts'].max()
-        updated_df['ts'] = updated_df['ts'].astype(int)
-        me_ts =updated_df['ts']
-        me_day_ts = me_ts-me_ts%(3600*24)
-        me_day_ts = me_day_ts.astype({'ts': 'int64'})
-        if me_day_ts.nunique()>1:
-            print('too many days saved together : investigate')
-        day_timestamp = me_day_ts.max()
-
-        day_dt_object = datetime.utcfromtimestamp(day_timestamp)
-        filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y')+'_dayohlc' #str(hourr_timestamp)
-        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
-        updated_df = updated_df.sort_values(by=['ts'])
-        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
-        updated_df = updated_df.set_index(updated_df['date'])
-        updated_df = updated_df.drop(columns=['date'])
-        self.day_df = updated_df
+        filename = self.update_signal_day_dataframe()
         local_path = self.local_root_directory + filename
-        self.day_df.to_csv(local_path)
+        print('uploading past days signals '+local_path)
+        self.day_signal_df.to_csv(local_path)
         self.aws_client.upload_file(self.bucket,local_path)
+
+        aggregated_local_path = local_path + 'discrete_aggregate'
+        print('uploading all discrete aggregated past daysfile '+aggregated_local_path)
+        pd.DataFrame(self.on_the_fly_discrete_day_signals).to_csv(aggregated_local_path)
+        self.aws_client.upload_file(self.bucket,aggregated_local_path)
         return
 
     def download_minute_dataframe(self, filename):
@@ -842,10 +1068,17 @@ class RealTimeSignalEnsemblingParalleLauncher():
                     if np.isnan(self.day_signal_np[-1, signal_counter]):
                         self.day_signal_np[-1, signal_counter] = 0
                 counter = counter + 1
-            last_signals = self.day_signal_np[-1, len(self.generic_columns) :]
-            return self.day_signals_ia_mixing(last_signals)
+            #updating the day signal df
+            self.update_signal_day_dataframe()
+            last_day_ts, discrete_day_signal = self.day_signals_ia_mixing()
+            self.update_day_aggregated_signal(last_day_ts, discrete_day_signal)
+            return discrete_day_signal
         return np.nan
 
+    def update_day_aggregated_signal(self, last_day_ts, discrete_day_signal):
+        new_day_sig_dic = {'ts': last_day_ts, 'day_discrete_sig': discrete_day_signal}
+        self.on_the_fly_discrete_minute_signals.append(new_day_sig_dic)
+
     def launchMinuteParallelPool(self, use_num_cpu):
         self.update_minute_dataframe()
         print('launching parallel computation for alphas at '+str(self.last_minute))
@@ -895,19 +1128,50 @@ class RealTimeSignalEnsemblingParalleLauncher():
                     if np.isnan(self.minute_signal_np[-1, signal_counter]):
                         self.minute_signal_np[-1, signal_counter] = 0
                 counter = counter + 1
-            last_signals = self.minute_signal_np[-1, len(self.generic_columns) :]
-            return self.minute_signals_ia_mixing(last_signals)
+            self.update_signal_minute_dataframe()
+            last_minute_ts, discrete_minute_signal = self.minute_signals_ia_mixing()
+            self.update_minute_aggregated_signal(last_minute_ts, discrete_minute_signal)
+            return discrete_minute_signal
         return np.nan
 
-    def day_signals_ia_mixing(self, signals):
-        nb_nan_signals = np.isnan(signals).sum()
-        print('still '+str(nb_nan_signals)+' not triggered day signals')
-        return np.nanmean(signals)
+    def update_minute_aggregated_signal(self, last_minute_ts, discrete_minute_signal):
+        new_minute_sig_dic = {'ts':last_minute_ts,'minute_discrete_sig':discrete_minute_signal}
+        self.on_the_fly_discrete_minute_signals.append(new_minute_sig_dic)
 
-    def minute_signals_ia_mixing(self, signals):
-        nb_nan_signals = np.isnan(signals).sum()
+
+
+    def day_signals_ia_mixing(self):
+        print('day signal shape')
+        print(self.day_signal_np.shape)
+        assert self.day_signal_np.shape[0] >= self.daily_ia_lookback_window
+        last_day_ts =  self.day_signal_np[-1, 0]
+        last_day_signals = self.day_signal_np[-1, len(self.generic_columns):]
+        nb_nan_signals = np.isnan(last_day_signals).sum()
+        print('still '+str(nb_nan_signals)+' not triggered day signals')
+        print('last daily alphas signal')
+        print(last_day_signals)
+        print('discretizing the signal')
+        if  self.day_signal_np.shape[0]> self.daily_ia_lookback_window + 1:
+            discrete_signals = self.daily_apply_standard_mean_two_states_discretization_last_signal()
+        else :
+            return np.nanmean(last_day_signals)
+        return last_day_ts, discrete_signals
+
+
+
+    def minute_signals_ia_mixing(self):
+        last_minute_ts =  self.minute_signal_np[-1, 0]
+        last_minute_signals = self.minute_signal_np[-1, len(self.generic_columns):]
+        nb_nan_signals = np.isnan(last_minute_signals).sum()
         print('still '+str(nb_nan_signals)+' not triggered minute signals')
-        return np.nanmean(signals)
+        print('last minute alphas signal')
+        print(last_minute_signals)
+        print('discretizing the signal')
+        if  self.minute_signal_np.shape[0]> self.minutely_ia_lookback_window + 1:
+            discrete_signals = self.minutely_apply_standard_mean_two_states_discretization_last_signal()
+        else :
+            return np.nanmean(last_minute_signals)
+        return last_minute_ts, discrete_signals
 
 
     def runMinuteSignal(self, me_signal):
